@@ -12,7 +12,7 @@ from multidict import MultiDict
 from url_normalize import url_normalize
 from yarl import URL
 
-RequestParams = Union[Mapping, Sequence, str, None]
+RequestParams = Union[Mapping[Any, Any], Sequence[Any], str, None]
 StrOrURL = Union[str, URL]
 
 __all__ = ["create_key"]
